<?php

namespace Drupal\tone\Form;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\BundleEntityFormBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;

/**
 * Form handler for tone type forms.
 */
class ToneTypeForm extends BundleEntityFormBase {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    /** @var \Drupal\tone\ToneTypeInterface $entity_type */
    $entity_type = $this->entity;
    if ($this->operation == 'edit') {
      $form['#title'] = $this->t('Edit %label tone type', ['%label' => $entity_type->label()]);
    }

    $form['label'] = [
      '#title' => $this->t('Label'),
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $entity_type->label(),
      '#description' => $this->t('The human-readable name of this tone type.'),
      '#size' => 30,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#required' => TRUE,
      '#default_value' => $entity_type->id(),
      '#maxlength' => EntityTypeInterface::BUNDLE_MAX_LENGTH,
      '#machine_name' => [
        'exists' => ['Drupal\tone\Entity\ToneType', 'load'],
        'source' => ['label'],
      ],
      '#description' => $this->t('A unique machine-readable name for this tone type. It must only contain lowercase letters, numbers, and underscores.'),
    ];

    $form['selector'] = [
      '#title' => $this->t('Selector'),
      '#type' => 'textfield',
      '#required' => TRUE,
      '#default_value' => $entity_type->getSelector(),
      '#description' => $this->t(
        'A CSS selector used to match elements on which the tone should be applied. The ampersand character (&) can be used in the same way as the <a href="https://sass-lang.com/documentation/style-rules/parent-selector">sass parent selector</a> to match the current entity.'
      ),
    ];

    $form['identity_plugin'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Identity Plugin'),
    ];

    $form['identity_plugin']['identity'] = [
      '#title' => $this->t('Identity'),
      '#type' => 'select',
      '#required' => TRUE,
      '#default_value' => $entity_type->getIdentityId(),
      '#options' => $this->getIdentityOptions(),
      '#empty_option' => $this->t('- Select -'),
      '#description' => $this->t('The identity plugin to use in order to generate CSS selectors for tones and data attributes for target entities.'),
      '#ajax' => [
        'callback' => '::updateIdentityConfiguration',
        'disable-refocus' => TRUE,
        'event' => 'change',
        'wrapper' => 'edit-tone-type-identity-configuration',
        'progress' => [
          'type' => 'throbber',
        ],
      ],
    ];

    $form['identity_plugin']['configuration'] = [
      '#type' => 'container',
      '#prefix' => '<div id="edit-tone-type-identity-configuration">',
      '#suffix' => '</div>',
    ];

    if ($entity_type->getIdentityId()) {
      $identity = $entity_type->getIdentity();
      $form['identity_plugin']['configuration']['about'] = [
        '#type' => 'item',
        '#weight' => -1,
        '#title' => $this->t('About the %plugin identity plugin', [
          '%plugin' => $identity->label(),
        ]),
        '#description' => $identity->getDescription(),
      ];
      if ($identity instanceof PluginFormInterface) {
        $form = NestedArray::mergeDeep($form, $identity->buildConfigurationForm($form, $form_state));
      }
    }

    $form['renderer_plugin'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Renderer Plugin'),
    ];

    $form['renderer_plugin']['renderer'] = [
      '#title' => $this->t('Renderer'),
      '#type' => 'select',
      '#required' => TRUE,
      '#default_value' => $entity_type->getRendererId(),
      '#options' => $this->getRendererOptions(),
      '#empty_option' => $this->t('- Select -'),
      '#description' => $this->t('The renderer plugin to use when generating source code for tone entities of this type.'),
      '#ajax' => [
        'callback' => '::updateRendererConfiguration',
        'disable-refocus' => TRUE,
        'event' => 'change',
        'wrapper' => 'edit-tone-type-renderer-configuration',
        'progress' => [
          'type' => 'throbber',
        ],
      ],
    ];

    $form['renderer_plugin']['configuration'] = [
      '#type' => 'container',
      '#prefix' => '<div id="edit-tone-type-renderer-configuration">',
      '#suffix' => '</div>',
    ];

    if ($entity_type->getRendererId()) {
      $renderer = $entity_type->getRenderer();
      $form['renderer_plugin']['configuration']['about'] = [
        '#type' => 'item',
        '#weight' => -1,
        '#title' => $this->t('About the %plugin renderer plugin', [
          '%plugin' => $renderer->label(),
        ]),
        '#description' => $renderer->getDescription(),
      ];
      if ($renderer instanceof PluginFormInterface) {
        $form = NestedArray::mergeDeep($form, $renderer->buildConfigurationForm($form, $form_state));
      }
    }

    $form['attachment_strategy_plugin'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Attachment Strategy Plugin'),
    ];

    $form['attachment_strategy_plugin']['attachment_strategy'] = [
      '#title' => $this->t('Attachment Strategy'),
      '#type' => 'select',
      '#required' => TRUE,
      '#default_value' => $entity_type->getAttachmentStrategyId(),
      '#options' => $this->getAttachmentStrategyOptions(),
      '#empty_option' => $this->t('- Select -'),
      '#description' => $this->t('The attachment strategy plugin to use when adding rendered code to a page.'),
      '#ajax' => [
        'callback' => '::updateAttachmentStrategyConfiguration',
        'disable-refocus' => TRUE,
        'event' => 'change',
        'wrapper' => 'edit-tone-type-attachment-strategy-configuration',
        'progress' => [
          'type' => 'throbber',
        ],
      ],
    ];

    $form['attachment_strategy_plugin']['configuration'] = [
      '#type' => 'container',
      '#prefix' => '<div id="edit-tone-type-attachment-strategy-configuration">',
      '#suffix' => '</div>',
    ];

    if ($entity_type->getAttachmentStrategyId()) {
      $strategy = $entity_type->getAttachmentStrategy();
      $form['attachment_strategy_plugin']['configuration']['about'] = [
        '#type' => 'item',
        '#weight' => -1,
        '#title' => $this->t('About the %plugin attachment strategy plugin', [
          '%plugin' => $strategy->label(),
        ]),
        '#description' => $strategy->getDescription(),
      ];
      if ($strategy instanceof PluginFormInterface) {
        $form = NestedArray::mergeDeep($form, $strategy->buildConfigurationForm($form, $form_state));
      }
    }

    return $this->protectBundleIdElement($form);
  }

  /**
   * Ajax callback for identity configuration.
   */
  public static function updateIdentityConfiguration(array $form, FormStateInterface $form_state) {
    return $form['identity_plugin']['configuration'];
  }

  /**
   * Ajax callback for renderer configuration.
   */
  public static function updateRendererConfiguration(array $form, FormStateInterface $form_state) {
    return $form['renderer_plugin']['configuration'];
  }

  /**
   * Ajax callback for renderer configuration.
   */
  public static function updateAttachmentStrategyConfiguration(array $form, FormStateInterface $form_state) {
    return $form['attachment_strategy_plugin']['configuration'];
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->t('Save tone type');
    $actions['delete']['#value'] = $this->t('Delete tone type');
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\tone\ToneTypeInterface $entity_type */
    $entity_type = $this->entity;

    if ($entity_type->getIdentityId()) {
      $identity = $entity_type->getIdentity();
      if ($identity instanceof PluginFormInterface) {
        $identity->validateConfigurationForm($form, $form_state);
      }
    }

    if ($entity_type->getRendererId()) {
      $renderer = $entity_type->getRenderer();
      if ($renderer instanceof PluginFormInterface) {
        $renderer->validateConfigurationForm($form, $form_state);
      }
    }

    if ($entity_type->getAttachmentStrategyId()) {
      $strategy = $entity_type->getAttachmentStrategy();
      if ($strategy instanceof PluginFormInterface) {
        $strategy->validateConfigurationForm($form, $form_state);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\tone\ToneTypeInterface $entity_type */
    $entity_type = $this->entity;

    $entity_type->set('id', trim($entity_type->id()));
    $entity_type->set('label', trim($entity_type->label()));
    $entity_type->set('selector', trim($entity_type->getSelector()));

    if ($entity_type->getIdentityId()) {
      $identity = $entity_type->getIdentity();
      if ($identity instanceof PluginFormInterface) {
        $identity->submitConfigurationForm($form, $form_state);
      }
    }

    if ($entity_type->getRendererId()) {
      $renderer = $entity_type->getRenderer();
      if ($renderer instanceof PluginFormInterface) {
        $renderer->submitConfigurationForm($form, $form_state);
      }
    }

    if ($entity_type->getAttachmentStrategyId()) {
      $strategy = $entity_type->getAttachmentStrategy();
      if ($strategy instanceof PluginFormInterface) {
        $strategy->submitConfigurationForm($form, $form_state);
      }
    }

    $status = $entity_type->save();

    $t_args = ['%name' => $entity_type->label()];
    if ($status == SAVED_UPDATED) {
      $message = $this->t('The tone type %name has been updated.', $t_args);
    }
    elseif ($status == SAVED_NEW) {
      $message = $this->t('The tone type %name has been added.', $t_args);
    }
    $this->messenger()->addStatus($message);

    $form_state->setRedirectUrl($entity_type->toUrl('collection'));
  }

  /**
   * Returns list of available renderer plugins.
   */
  protected function getIdentityOptions(): array {
    $manager = \Drupal::service('plugin.manager.tone_identity');
    return array_map(
      fn ($definition) => $definition['label'],
      $manager->getDefinitions()
    );
  }

  /**
   * Returns list of available renderer plugins.
   */
  protected function getRendererOptions(): array {
    $manager = \Drupal::service('plugin.manager.tone_renderer');
    return array_map(
      fn ($definition) => $definition['label'],
      $manager->getDefinitions()
    );
  }

  /**
   * Returns list of available attachment strategy plugins.
   */
  protected function getAttachmentStrategyOptions(): array {
    $manager = \Drupal::service('plugin.manager.tone_attachment_strategy');
    return array_map(
      fn ($definition) => $definition['label'],
      $manager->getDefinitions()
    );
  }

}
